*=============================================================================*
*
*  File YC_RULER:  On-Screen row/column ruler.
*                  Can be called at anytime by assigning to a key with SET KEY.
*                  Great tool for coding complicated @..SAY..GET screens. 
*
*    Programmer:  Craig Yellick                 Source  BEB817
*                 Yellick Computing
*                 7 West Fourth Street
*                 St. Paul, Minnesota 55102
*                 (612) 291-7594
*
*  Placed in the public domain with no restrictions on August 26, 1987.
*  This function first appeared in the Oct '87 issue of Reference(Clipper).
*
*=============================================================================*
*
*  Usage:  Add the line "Set Key nnn TO Ruler" to your main procedure, where
*          nnn is the INKEY() value of the key to use to invoke the ruler.
*          For example, use the value -20 to assign the ruler to CTRL-F1.
*          Make sure the OBJ version of this file is linked into your
*          application.
*
*          The following keys are used to control the ruler-
*
*              Up Down Left Right  -  Take a wild guess
*              Home End            -  Upper left and lower right of screen
*              Space-Bar           -  Return to center of screen
*              Plus (+)            -  Toggle cursor type
*              Tab                 -  Toggle offset on/off
*              1..9                -  Movement multiplier for rapid positioning
*                                     (Primarily for column movement, row
*                                     movement will stay somewhat proportional)
*              Esc                 -  Exit
*
*          Line zero will contain a status display:
*
*                              Ruler @ r,c  Move xN
*
*          Where r = row, c = column and N is the movement multiplier.
*          If the offset toggle is on an additional phrase is displayed:
*
*                                   Offset x,y
*
*          Where x = rows, x = columns relative to the main cursor position.
*          This is useful for calculating relative coordinates for boxes.
*          The offset ruler is turned on and off with the TAB key.
*
*  Notes:  This procedure calls the assembly-language functions CURSON and
*          CURSOFF to get rid of the cursor.  If you do not have access to
*          equivalent cursor control functions you can delete both calls, the 
*          screen will just be a little noisy with cursor flashes.  Or, you
*          may have to substitute the calls to a cursor function you presently
*          use, I have seen some like CALL CURSOR WITH "OFF" etc.
*
*          I have used BORING black & white SET COLOR TO commands so this
*          procedure will work well on all monitors.  For best results and
*          a stunning effect, try changing the color of the various cursor
*          types and lines.  If you have access to a routine that changes the
*          background color of a screen area without disturbing the characters
*          you could make some great improvements as well.
*
*=============================================================================*

PROCEDURE RULER

*
* Don't want to disturb the rest of the system...
*
PRIVATE cr, cc, r, k, redraw, crosshairs, offset, scr_ruler
PRIVATE rinit, cinit, rmax, cmax, rmin, cmin, rmult, cmult, rcur, ccur

cr= ROW()  && Save current cursor row and column so we
cc= COL()  && can restore the screen exactly as we found it

CALL CursOff               && Get rid of the cursor
SAVE SCREEN TO scr_ruler   && Save the current screen

*
* Define Constants  (These make the code easier to read)
*
rmin=  0                 && Minimum row
cmin=  0                 && Minimum column
rmax=  24                && Maximum row
cmax=  79                && Maximum column
rinit= rmax /2           && Initial row
cinit= cmax /2           && Initial column

*
* Initialize Variables
*
rcur=        rinit       && Current row
ccur=        cinit       && Current column
xcur=        rinit       && Current offset row
ycur=        cinit       && Current offset column
rmult=       1           && Row movement multiplier
cmult=       1           && Column movement multiplier
redraw=     .t.          && Screen redraw flag
crosshairs= .t.          && Cross hairs/block display toggle
offset=     .f.          && Offset cursor toggle

*
*  Let's Do It To It
*
DO WHILE .t.  && We'll in stay here until ESC key is pressed

  *
  *  Screen Redraw
  *
  IF redraw
    redraw= .f.
    RESTORE SCREEN FROM scr_ruler  && Bring back the original screen

    @ 0,0                          && Clear the status line and redisplay
    @ 0,0 SAY "Ruler @"
    @ 0, COL()+1 SAY LTRIM(STR(rcur, 2)) +","
    @ 0, COL()+0 SAY LTRIM(STR(ccur, 2))
    IF offset
      @ 0, COL()+2 SAY "Offset"
      @ 0, COL()+1 SAY LTRIM(STR(xcur -rcur, 3)) +","
      @ 0, COL()+0 SAY LTRIM(STR(ycur -ccur, 3))
    ENDIF
    @ 0, COL()+2 SAY "Move x"
    @ 0, COL()+0 SAY LTRIM(STR(cmult,2))

    *
    *  Display either cross-hair cursor or a flashing block
    *
    IF crosshairs
      DO CASE

      *  We're in the first column
      CASE ccur = cmin
        @ rcur, cmin +1 SAY REPLICATE(CHR(250), cmax -1) +CHR(180)

      *  We're in the last column
      CASE ccur = cmax
        @ rcur, cmin SAY CHR(195) +REPLICATE(CHR(250), cmax -1)

      *  We're somewhere in between
      OTHERWISE
        @ rcur, cmin SAY CHR(195) +REPLICATE(CHR(250), ccur -1)
        @ rcur, ccur +1 SAY REPLICATE(CHR(250), cmax -(ccur +1)) +CHR(180)
      ENDCASE

      *  If we're not on the first row
      IF rcur <> rmin
        @ rmin, ccur SAY CHR(194)
      ENDIF

			*  Do the middle of the line
      r= rmin +1
      DO WHILE r < rmax
        @ r, ccur SAY IIF(r = rcur, [], CHR(250))   && Skip our current pos
        r= r +1
      ENDDO

      *  If we're not on the last row
      IF rcur <> rmax
        @ rmax, ccur SAY CHR(193)
      ENDIF
  
    ELSE                        && Flashing block, instead
      SET COLOR TO W*/N
      @ rcur, ccur SAY CHR(254)
      SET COLOR TO W/N

    ENDIF && Drawing the cross-hairs

    *
    *  Display offset cursor if needed
    *
    IF offset
      SET COLOR TO W*/N
      @ xcur, ycur SAY CHR(197)
      SET COLOR TO W/N
    ENDIF

  ENDIF && Redraw

  k= INKEY()   &&  Get a keypress
  
  redraw= .t.  &&  Assume we'll have to redraw the screen

  *
  *  Interpret the key
  *
  DO CASE
  CASE k = 27                     && Escape
    EXIT
  CASE k = 43                     && (+) Toggle cross hairs
    crosshairs= .not. crosshairs
  CASE k =  9                     && (Tab) Toggle offset cursor 
    offset= .not. offset
  CASE k =  5                     && Row Up
    IF offset
      xcur= xcur -rmult
    ELSE
      rcur= rcur -rmult
    ENDIF
  CASE k = 24                     && Row Down
    IF offset
      xcur= xcur +rmult
    ELSE
      rcur= rcur +rmult
    ENDIF
  CASE k = 19                     && Column Left
    IF offset
      ycur= ycur -cmult
    ELSE
      ccur= ccur -cmult
    ENDIF
  CASE k =  4                     && Column Right
    IF offset
      ycur= ycur +cmult
    ELSE
      ccur= ccur +cmult
    ENDIF
  CASE k =  32                    && (Space Bar) Center of Screen
    IF offset
      xcur= rinit
      ycur= cinit
    ELSE
      rcur= rinit
      ccur= cinit
    ENDIF
  CASE k =  1                     && (Home) Upper Left Corner
    IF offset
      xcur= rmin
      ycur= cmin
    ELSE
      rcur= rmin
      ccur= cmin
    ENDIF
  CASE k =  6                     && (End) Lower Right Corner
    IF offset
      xcur= rmax
      ycur= cmax
    ELSE
      rcur= rmax
      ccur= cmax
    ENDIF
  CASE (k > 48) .AND. (k < 58)    && (Keys 1..9) Move Multiplier
    cmult= k -48
    *
    *  Make row movement proportional to column
    *  but be sure it stays at least one
    *
    rmult= MAX(1, ROUND(cmult *(rmax/cmax),0) )

  OTHERWISE
    redraw= .f.   &&  Keypress did not require screen redraw

  ENDCASE  && Interpreting keypress

  *
  * Perform row/col range checking, wrap around if necessary
  *
  rcur= Wrap(rcur, rmin, rmax)
  ccur= Wrap(ccur, cmin, cmax)
  xcur= Wrap(xcur, rmin, rmax)
  ycur= Wrap(ycur, cmin, cmax)

ENDDO  && While Escape not pressed

RESTORE SCREEN FROM scr_ruler      && All done, restore original screen
CALL CursOn                        && and turn the cursor back on,         <<<<
@ cr, cc SAY []                    && parked back where we found it

RETURN  && End Procedure Ruler

FUNCTION   Wrap  && Wrap cursor postion between min and max values
PARAMETERS cur, min, max
RETURN     IIF(cur > max, min, IIF(cur < min, max, cur))

* EOF YC_RULER.PRG
